#ifdef _AUDIO
VVAR(unsigned char	*,_l2u,V__l2u)
VVAR(short		*,_u2l,V__u2l)
VFUNC(void,Audio_Load,V_Audio_Load,_((Audio *au, InputStream f)))
VFUNC(void,Audio_Save,V_Audio_Save,_((Audio *au, OutputStream f, char *comment)))
VFUNC(void,Audio_autocorrelation,V_Audio_autocorrelation,_((int N, float *x,unsigned p,float *r)))
VFUNC(void,Audio_complex_debug,V_Audio_complex_debug,_((int N,float *x,PerlIO *f)))
VFUNC(void,Audio_conjugate,V_Audio_conjugate,_((int n,float *data,float scale)))
VFUNC(void,Audio_difference,V_Audio_difference,_((int n, float *a, float *b)))
VFUNC(void,Audio_durbin,V_Audio_durbin,_((int NUM_POLES, float *R,float *aa)))
VFUNC(int,Audio_lpc,V_Audio_lpc,_((int length, const float *sig, int order, float *acf,
   	              float *ref, float *lpc)))
VFUNC(float *,Audio_more,V_Audio_more,_((pTHX_ Audio *au, int n)))
VFUNC(void,Audio_r2_fft,V_Audio_r2_fft,_((int n,float *data)))
VFUNC(void,Audio_r2_ifft,V_Audio_r2_ifft,_((int n,float *data)))
VFUNC(void,Audio_r4_fft,V_Audio_r4_fft,_((int n,float *data)))
VFUNC(void,Audio_r4_ifft,V_Audio_r4_ifft,_((int n,float *data)))
VFUNC(IV,Audio_rate,V_Audio_rate,_((Audio * au, IV rate)))
VFUNC(SV *,Audio_shorts,V_Audio_shorts,_((Audio *au)))
VFUNC(float *,Audio_w,V_Audio_w,_((int n)))
VFUNC(long,float2linear,V_float2linear,_((float f,int bits)))
VFUNC(long,float2ulaw,V_float2ulaw,_((float f)))
VFUNC(float,linear2float,V_linear2float,_((long l,int bits)))
VFUNC(float,ulaw2float,V_ulaw2float,_((long u)))
#endif /* _AUDIO */
